/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_INNOLIDARSERVICE_INNOLIDARSERVICE_SKELETON_H
#define JAC_AP_INNOLIDARSERVICE_INNOLIDARSERVICE_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/innolidarservice/innolidarservice_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace innolidarservice {
namespace skeleton {
namespace events
{
    using InnoLiDAR2LiDARPerceptionEvent = ara::com::internal::skeleton::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    static constexpr ara::com::internal::EntityId InnoLiDARServiceInnoLiDAR2LiDARPerceptionEventId = 57993U; //InnoLiDAR2LiDARPerceptionEvent_event_hash
}

namespace methods
{
}

namespace fields
{
}

class InnoLiDARServiceSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultInnoLiDAR2LiDARPerceptionEvent = InnoLiDAR2LiDARPerceptionEvent.Initialize();
        ThrowError(resultInnoLiDAR2LiDARPerceptionEvent);
    }

    InnoLiDARServiceSkeleton& operator=(const InnoLiDARServiceSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instanceId, mode)),
              InnoLiDAR2LiDARPerceptionEvent(ptr->GetSkeleton(), events::InnoLiDARServiceInnoLiDAR2LiDARPerceptionEventId, ::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instanceSpec, mode)),
              InnoLiDAR2LiDARPerceptionEvent(ptr->GetSkeleton(), events::InnoLiDARServiceInnoLiDAR2LiDARPerceptionEventId, ::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instanceContainer, mode)),
              InnoLiDAR2LiDARPerceptionEvent(ptr->GetSkeleton(), events::InnoLiDARServiceInnoLiDAR2LiDARPerceptionEventId, ::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              InnoLiDAR2LiDARPerceptionEvent(std::move(other.InnoLiDAR2LiDARPerceptionEvent)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                InnoLiDAR2LiDARPerceptionEvent = std::move(other.InnoLiDAR2LiDARPerceptionEvent);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::InnoLiDAR2LiDARPerceptionEvent GetInnoLiDAR2LiDARPerceptionEvent()
        {
            return std::move(InnoLiDAR2LiDARPerceptionEvent);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = InnoLiDAR2LiDARPerceptionEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::InnoLiDAR2LiDARPerceptionEvent InnoLiDAR2LiDARPerceptionEvent;
    };
    explicit InnoLiDARServiceSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instanceId, mode)),
          InnoLiDAR2LiDARPerceptionEvent(skeletonAdapter->GetSkeleton(), events::InnoLiDARServiceInnoLiDAR2LiDARPerceptionEventId, ::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit InnoLiDARServiceSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instanceSpec, mode)),
          InnoLiDAR2LiDARPerceptionEvent(skeletonAdapter->GetSkeleton(), events::InnoLiDARServiceInnoLiDAR2LiDARPerceptionEventId, ::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit InnoLiDARServiceSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instanceContainer, mode)),
          InnoLiDAR2LiDARPerceptionEvent(skeletonAdapter->GetSkeleton(), events::InnoLiDARServiceInnoLiDAR2LiDARPerceptionEventId, ::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    InnoLiDARServiceSkeleton(const InnoLiDARServiceSkeleton&) = delete;

    InnoLiDARServiceSkeleton(InnoLiDARServiceSkeleton&&) = default;
    InnoLiDARServiceSkeleton& operator=(InnoLiDARServiceSkeleton&&) = default;
    InnoLiDARServiceSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          InnoLiDAR2LiDARPerceptionEvent(token.GetInnoLiDAR2LiDARPerceptionEvent()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~InnoLiDARServiceSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&InnoLiDARServiceSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::InnoLiDAR2LiDARPerceptionEvent InnoLiDAR2LiDARPerceptionEvent;
};
} // namespace skeleton
} // namespace innolidarservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_INNOLIDARSERVICE_INNOLIDARSERVICE_SKELETON_H
